var scale = 1,
  scale_down_limit = 0.25,
  scale_up_limit = 5,
  canvas = document.querySelector('#pdf-render'),
  ctx = canvas.getContext('2d'),

  curr = new Object,
  pdf_1st = new Object,
  pdf_2nd = new Object,
  pdf_new = new Object,

  pageIsRendering = false,
  pageNumIsPending = null,

  should_overlay_showing = false;

const queueRenderPage = p_doc => {
  if (curr.pageIsRendering) {
    pageNumIsPending = p_doc.pageNum;
  } else {
    renderPage(p_doc, p_doc.pageNum);
  }
};

// Show Prev Page
const showPrevPage = () => {
  if (curr.pageNum <= 1) {
    return;
  }
  curr.pageNum--;
  renderPageInMain();
};

// Show Next Page
const showNextPage = () => {
  if (curr.pageNum >= curr.numPages) {
    return;
  }
  curr.pageNum++;
  renderPageInMain();

//    renderPage(f1, 2);
};

// Button Events
document.querySelector('#prev-page').addEventListener('click', showPrevPage);
document.querySelector('#next-page').addEventListener('click', showNextPage);
document.querySelector('#scale-up').addEventListener('click', ()=>{if(scale>scale_up_limit) return;scale*=1.2; renderPageInMain();});
document.querySelector('#scale-down').addEventListener('click', ()=>{if(scale<scale_down_limit) return;scale*=0.8; renderPageInMain();});


const get_pdf_by_url = (url, div_id) => {
    var pdf = new Object();
    pdf.url = url;
//    pdf.pageIsRendering = false;
    pdfjsLib.getDocument(url).promise.then(pdfDoc_ => {
    pdf.pdfDoc = pdfDoc_;
    pdf.numPages = pdf.pdfDoc.numPages;
    pdf.pageNum = 1;
    curr = pdf;

    if(div_id == '1'){
        pdf_1st = pdf;
    } else if(div_id == '2'){
        pdf_2nd = pdf;
    } else if(div_id == 'new'){
        pdf_new = pdf;
    }
//    renderPage(pdf, pdf.pageNum);
    renderPageInMain();
//    console.log(pdf.url);

    generate_preview(pdf, div_id);
    should_overlay_showing = false;
  })
  .catch(err => {
  should_overlay_showing = false;
    // Display error
//    const div = document.createElement('div');
//    div.className = 'error';
//    div.appendChild(document.createTextNode(err.message));
//    document.querySelector('body').insertBefore(div, canvas);
//    // Remove top bar
//    document.querySelector('.top-bar').style.display = 'none';
  });
  return pdf;
};

const renderPage = (pdf, num) => {
  pageIsRendering = true;

  // Get page
  pdf.pdfDoc.getPage(num).then(page => {
    // Set scale
    const viewport = page.getViewport({ scale });
    canvas.height = viewport.height;
    canvas.width = viewport.width;

    const renderCtx = {
      canvasContext: ctx,
      viewport
    };

    page.render(renderCtx).promise.then(() => {
      pdf.pageIsRendering = false;

      if (pageNumIsPending !== null) {
        renderPage(curr, pageNumIsPending);
        pageNumIsPending = null;
      }
    });
    should_overlay_showing = false;
  });
};

const renderPageInMain = () => {
    // Output current page
    document.querySelector('#page-num').textContent = curr.pageNum;
    document.querySelector('#page-count').textContent = curr.numPages;
    renderPage(curr, curr.pageNum);
    should_overlay_showing = false;
}

// Check for the various File API support.
// https://www.html5rocks.com/en/tutorials/file/dndfiles/
if (window.File && window.FileReader && window.FileList && window.Blob) {
  // Great success! All the File APIs are supported.
} else {
  alert('The File APIs are not fully supported in this browser.');
}

function handleFileSelect(evt) {
    evt.stopPropagation();
    evt.preventDefault();

//    var f = evt.dataTransfer.files[0]; // FileList object.
    var f = new Object();
    if ($(evt.target).hasClass('file-drop')){
        f = evt.originalEvent.dataTransfer.files[0];
    }
    if ($(evt.target).hasClass('file-select')){
        f = evt.target.files[0];
    }
//    $('#file_1st_name')[0].textContent = escape(f.name);
//  find top div id
    var top_div_id = $(evt.target.closest('div.card')).prop('id');
    $('#'+top_div_id).find('button.file-name').prop('textContent', f.name);
//    $('#file_1st_name')[0].textContent = f.name;

    show_overlay_bar();

    var reader = new FileReader();
    // Closure to capture the file information.
    reader.onload = (function(theFile) {
        return async function(e) {
        let url = await eel.upload_pdf(f.name, e.target.result)();
        get_pdf_by_url(url, top_div_id);
        };
      })(f);

    // Read in the image file as a data URL.
    reader.readAsDataURL(f);
//    reader.readAsArrayBuffer(f);
}

function handleDragOver(evt) {
    evt.stopPropagation();
    evt.preventDefault();
    evt.originalEvent.dataTransfer.dropEffect = 'copy'; // Explicitly show this is a copy.
}

// Setup the dnd listeners.
//var dropZone = document.getElementById('drop_zone');
//dropZone.addEventListener('dragover', handleDragOver, false);
//dropZone.addEventListener('drop', handleFileSelect, false);
//document.getElementById('file_1st').addEventListener('change', handleFileSelect1, false);
$('div.file-drop').on('dragover', handleDragOver);
$('div.file-drop').on('drop', handleFileSelect);
$('input.file-select').on('change', handleFileSelect);

function page_click(evt){
    [f, page] = $(evt.target).parent('.input-group').prop('id').split('-');
    if(f == '1'){
        curr = pdf_1st;
    } else if(f=='2'){
        curr = pdf_2nd;
    } else if(f=='new'){
        let [index, p]= $(evt.target).parent('.input-group').data('i-n');
        page = p;
        if(index=='1'){curr = pdf_1st;}
        if(index=='2'){curr = pdf_2nd;}
    }
    curr.pageNum = parseInt(page);
    renderPageInMain();
}

function generate_preview(pdf, div_id){
    let div = $('#'+div_id).find('.preview-group');
    div.empty();
    for (i = 1; i <= pdf.numPages; i++) {
    let s = $('#preview-item-template').clone();
//    s.removeClass('invisible');
        s.prop('id', div_id+'-'+i);

        div.append(s.prop('outerHTML'));
        $('#'+div_id+'-'+i+' .form-control').prop('value', i+'');
    }

    div.find('input.page-num').on('click', page_click);
}

function select_all_page(evt){
    $(evt.target).parents('div.collapse').find('.page-num-check').prop('checked', true);
}
$('button.select-all').on('click', select_all_page);
function deselect_all_page(evt){
    $(evt.target).parents('div.collapse').find('.page-num-check').prop('checked', false);
}
$('button.deselect-all').on('click', deselect_all_page);

async function save_as(evt){
    let name_page_list = [];
    if($(evt.target).parents('.card').prop('id') !== 'new'){
        let name = $(evt.target).parents('.card').find('.file-name').prop('textContent');
        $(evt.target).parents('.card').find('.preview-group .input-group').each(function(){
            if ($(this).find('.page-num-check').prop('checked')){
                name_page_list.push([name, $(this).find('.form-control').prop('value')]);
            }
        });
    } else {
        $('#new .preview-group .input-group').each(function(){
            name_page_list.push($(this).data('f-n'));
        });
    }

    let r = await eel.save_as_pdf(name_page_list)();
    console.log(r);
//    window.open('/res/'+escape(name) , '_blank');
    var a = document.createElement("a");
    a.href = escape(r);
    a.setAttribute("download", name);
    a.click();
}
$('button.save-as').on('click', save_as);

async function add_to_new(evt){
    let name = $(evt.target).parents('.card').find('.file-name').prop('textContent');
    let index = $(evt.target).parents('.card').prop('id');
    let name_page_list = [];
    let index_page_list = [];

    $(evt.target).parents('.card').find('.preview-group .input-group').each(function(){
        if ($(this).find('.page-num-check').prop('checked')){
            let p = $(this).find('.form-control').prop('value');
            name_page_list.push([name, p]);
            index_page_list.push([index, p]);
        }
    });

    let div = $('#preview-group-new');
    let existing_new = 0;
    if ($('#preview-group-new > .input-group').length == 0){
        $('#preview-group-new').data('serial', name_page_list.length);
    } else {
        existing_new = $('#preview-group-new').data('serial');
        $('#preview-group-new').data('serial', existing_new+name_page_list.length);
    }
    for(i=0; i<name_page_list.length; i++){
        let s = $('#preview-new-template').clone();
    //    s.removeClass('invisible');
        s.prop('id', 'new-'+(i+existing_new));

        div.append(s.prop('outerHTML'));
        $('#new-'+(i+existing_new)+' .form-control').prop('value', name_page_list[i][1]+':' + name_page_list[i][0]);
        $('#new-'+(i+existing_new)).data('f-n', name_page_list[i]);
        $('#new-'+(i+existing_new)).data('i-n', index_page_list[i]);
    }
    $('#new input.page-num').off('click').on('click', page_click);
//    $('#new input.page-num').on('click', page_click);
    $('#new .card-header button.btn').click();

    var source = $('#new .preview-group-new .input-group'),
    dragElement = null,                                         // 用于存放拖动元素
    lock = true;                                                // 最后元素拖放拖放时会进入enter和leave的循环，用来锁住
//    source.off('dragstart').on('dragstart', function(ev){
//        dragElement = $(ev.target);                                     // 用于存放拖动元素
////        this.style.backgroundColor = '#f8f8f8';                 // 设置拖动元素的背景
//        $(ev.target).style.backgroundColor = 'olivedrab';                 // 设置拖动元素的背景
//    },false);
//    source.off('dragend').on('dragend', function(ev){
//        ev.target.style.backgroundColor = '#fff';               // 拖放结束还原拖动元素的背景
//        ev.preventDefault();
//    },false);
//    source.off('dragenter').on('dragenter', function(ev){
//        if(dragElement != $(ev.target)){
//            $(ev.target).parentNode.insertBefore(dragElement,$(ev.target));     // 把拖动元素添加到当前元素的前面
//        }
//    }, false);
//    source.off('dragleave').on('dragleave', function(ev){
//        if(dragElement != $(ev.target)){
//            if(lock && ($(ev.target) == $(ev.target).parentNode.lastElementChild || $(ev.target) == $(ev.target).parentNode.lastChild)){    // 当前元素时最后一个元素
//                $(ev.target).parentNode.appendChild(dragElement);       // 把拖动元素添加最后面
//                lock = false;
//            }else{
//                lock = true;
//            }
//        }
//    }, false);

for(var i = 0; i < source.length; i++){
    source[i].addEventListener('dragstart',function(ev){
        dragElement = this;                                     // 用于存放拖动元素
//        this.style.backgroundColor = '#f8f8f8';                 // 设置拖动元素的背景
        this.style.backgroundColor = 'olivedrab';                 // 设置拖动元素的背景
    },false);

    source[i].addEventListener('dragend',function(ev){
        ev.target.style.backgroundColor = '#fff';               // 拖放结束还原拖动元素的背景
        ev.preventDefault();
    },false)

    source[i].addEventListener('dragenter', function(ev){
        if(dragElement != this){
            this.parentNode.insertBefore(dragElement,this);     // 把拖动元素添加到当前元素的前面
        }
    }, false)

    source[i].addEventListener('dragleave', function(ev){
        console.log(111);
        if(dragElement != this){
            if(lock && (this == this.parentNode.lastElementChild || this == this.parentNode.lastChild)){    // 当前元素时最后一个元素
                this.parentNode.appendChild(dragElement);       // 把拖动元素添加最后面
                lock = false;
            }else{
                lock = true;
            }
        }
    }, false)
};

}
$('button.add-to').on('click', add_to_new);

function delete_from_new(evt){
    $('#new .preview-group .input-group').each(function(){
        if ($(this).find('.page-num-check').prop('checked')){
            $(this).remove();
        }
    });
}
$('button.delete').on('click', delete_from_new)

function show_overlay_bar(){
    $('#progress-bar').modal('show');
    should_overlay_showing = true;
}
function hide_overlay_bar(){
    if(should_overlay_showing == false){
        $('#progress-bar').modal('hide');
        }
}
setInterval(hide_overlay_bar, 1000); // Time in milliseconds

document.ondragover = function(e){e.preventDefault();}          // 必须设置dragover阻止默认事件
document.ondrop = function(e){e.preventDefault();}

$('#about').on('click',()=>{window.open('http://qishansun.com');});